"use strict";
/**
 * Serverless - Ajax
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.Ajax = void 0;
const config_js_1 = require("../../config/config.js");
const action_js_1 = require("../../utils/action/action.js");
const shortcode_js_1 = require("../../utils/shortcode/shortcode.js");
const filter_js_1 = require("../../utils/filter/filter.js");
const object_js_1 = require("../../utils/object/object.js");
const string_js_1 = require("../../utils/string/string.js");
const number_js_1 = require("../../utils/number/number.js");
const print_js_1 = require("../../utils/print/print.js");
const path_js_1 = require("../../utils/path/path.js");
const ResponseError_js_1 = require("../../utils/ResponseError/ResponseError.js");
const SendForm_js_1 = require("../SendForm/SendForm.js");
/**
 * Set env variables, normalize request body, check for required props and call actions
 *
 * @param {AjaxArgs} args
 * @return {Promise<Response>} Response
 */
const Ajax = async ({ request, functionPath, env, siteConfig }) => {
    try {
        /* Config */
        (0, config_js_1.setConfig)(siteConfig);
        await (0, config_js_1.setConfigFilter)(env);
        siteConfig.env.dir = (0, path_js_1.getPathDepth)(functionPath);
        (0, filter_js_1.setFilters)(siteConfig.filters);
        (0, action_js_1.setActions)(siteConfig.actions);
        (0, shortcode_js_1.setShortcodes)(siteConfig.shortcodes);
        /* Get form data */
        const data = await request.json();
        /* Data must be object */
        if (!(0, object_js_1.isObjectStrict)(data)) {
            throw new Error('Data not an object');
        }
        /* Inputs required */
        if (data.inputs === undefined) {
            throw new Error('No inputs');
        }
        /* Honeypot check */
        const honeypotName = `${siteConfig.namespace}_asi`;
        if (data.inputs[honeypotName] !== undefined) {
            const honeypotValue = data.inputs[honeypotName].value;
            if ((0, string_js_1.isStringStrict)(honeypotValue)) {
                const options = {
                    status: 200,
                    headers: {
                        'Content-Type': 'application/json'
                    }
                };
                return new Response(JSON.stringify({ success: 'Form successully sent' }), options);
            }
            data.inputs[honeypotName].exclude = true;
        }
        /* Id required */
        if (!(0, string_js_1.isStringStrict)(data.id)) {
            throw new Error('No id');
        }
        /* Action required */
        const action = data.action !== undefined ? data.action : '';
        if (!(0, string_js_1.isStringStrict)(action)) {
            throw new Error('No action');
        }
        /* Call functions by action */
        let res = null;
        if (action === 'sendForm') {
            res = await (0, SendForm_js_1.SendForm)({ ...data, env, request });
        }
        if (siteConfig.ajaxFunctions[action] !== undefined) {
            res = await siteConfig.ajaxFunctions[action]({ ...data, env, request });
        }
        const ajaxResFilterArgs = {
            ...data,
            action,
            env,
            request
        };
        res = await (0, filter_js_1.applyFilters)('ajaxRes', res, ajaxResFilterArgs);
        /* Result error */
        if (res === null) {
            throw new Error('No result');
        }
        if (res.error !== undefined) {
            throw new ResponseError_js_1.ResponseError(res.error.message, res.error.resp);
        }
        /* Result success */
        const options = {
            status: 200,
            headers: {
                'Content-Type': 'application/json'
            }
        };
        let message = '';
        if (res.success !== undefined) {
            const { message: successMessage, headers } = res.success;
            if (successMessage !== undefined) {
                message = successMessage;
            }
            if (headers !== undefined) {
                options.headers = { ...options.headers, ...headers };
            }
        }
        return new Response(JSON.stringify({ success: message }), options);
    }
    catch (error) {
        (0, print_js_1.print)('[SSF] Error with ajax function', error);
        let statusCode = 500;
        let message = '';
        if (error instanceof ResponseError_js_1.ResponseError) {
            if ((0, number_js_1.isNumber)(error.response?.status)) {
                statusCode = error.response.status;
            }
            if ((0, string_js_1.isStringStrict)(error.message)) {
                message = error.message;
            }
        }
        const options = {
            status: statusCode,
            headers: {
                'Content-Type': 'application/json'
            }
        };
        return new Response(JSON.stringify({ error: message }), options);
    }
};
exports.Ajax = Ajax;
