"use strict";
/**
 * Utils - Action
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.setActions = exports.resetActions = exports.doActions = exports.removeAction = exports.addAction = void 0;
const string_js_1 = require("../string/string.js");
const array_js_1 = require("../array/array.js");
const object_js_1 = require("../object/object.js");
const function_js_1 = require("../function/function.js");
/**
 * Store action callbacks by name
 *
 * @type {ActionsFunctions}
 */
let actions = {
    renderStart: [],
    renderEnd: [],
    renderItemStart: [],
    renderItemEnd: []
};
/**
 * Add action to action object
 *
 * @param {string} name
 * @param {function} action
 * @return {boolean}
 */
const addAction = (name, action) => {
    if (!(0, string_js_1.isStringStrict)(name) || !(0, function_js_1.isFunction)(action)) {
        return false;
    }
    if (actions[name] === undefined) {
        actions[name] = [];
    }
    actions[name].push(action);
    return true;
};
exports.addAction = addAction;
/**
 * Remove action from actions object
 *
 * @param {string} name
 * @param {function} action
 * @return {boolean}
 */
const removeAction = (name, action) => {
    if (!(0, string_js_1.isStringStrict)(name) || !(0, function_js_1.isFunction)(action)) {
        return false;
    }
    const callbacks = actions[name];
    if ((0, array_js_1.isArrayStrict)(callbacks)) {
        const index = callbacks.indexOf(action);
        if (index > -1) {
            callbacks.splice(index, 1);
            return true;
        }
    }
    return false;
};
exports.removeAction = removeAction;
/**
 * Run callback functions from actions object
 *
 * @param {string} name
 * @param {*} [args]
 * @return {Promise<void>}
 */
const doActions = async (name, args) => {
    const callbacks = actions[name];
    if ((0, array_js_1.isArrayStrict)(callbacks)) {
        for (const callback of callbacks) {
            if ((0, function_js_1.isFunction)(callback)) {
                await callback(args);
            }
        }
    }
};
exports.doActions = doActions;
/**
 * Empty actions object
 *
 * @return {void}
 */
const resetActions = () => {
    actions = {
        renderStart: [],
        renderEnd: [],
        renderItemStart: [],
        renderItemEnd: []
    };
};
exports.resetActions = resetActions;
/**
 * Fill actions object
 *
 * @param {Actions} args
 * @return {boolean}
 */
const setActions = (args) => {
    if (!(0, object_js_1.isObjectStrict)(args)) {
        return false;
    }
    const newActions = Object.entries(args);
    if (newActions.length === 0) {
        return false;
    }
    resetActions();
    newActions.forEach(([name, action]) => {
        if (action === undefined) {
            return;
        }
        addAction(name, action);
    });
    return true;
};
exports.setActions = setActions;
