"use strict";
/**
 * Utils - File Data
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAllFileData = exports.getFileData = void 0;
const promises_1 = require("node:fs/promises");
const node_path_1 = require("node:path");
const filter_js_1 = require("../filter/filter.js");
const object_js_1 = require("../object/object.js");
const string_js_1 = require("../string/string.js");
const json_js_1 = require("../json/json.js");
const print_js_1 = require("../print/print.js");
const config_js_1 = require("../../config/config.js");
const fileDataNormal_js_1 = require("./fileDataNormal.js");
const array_js_1 = require("../array/array.js");
const function_js_1 = require("../function/function.js");
/**
 * Get data from file or cache
 *
 * @param {string} key
 * @param {FileDataParams} params
 * @return {Promise<FileDataReturn>}
 */
const getFileData = async (key = '', params = {}) => {
    try {
        /* Key required for cache */
        if (!(0, string_js_1.isStringStrict)(key)) {
            throw new Error('No key');
        }
        /* Check cache */
        if (config_js_1.config.env.cache) {
            let cacheData = {};
            const cacheDataFilterArgs = {
                key,
                type: 'get'
            };
            cacheData = await (0, filter_js_1.applyFilters)('cacheData', cacheData, cacheDataFilterArgs);
            if ((0, object_js_1.isObject)(cacheData) && Object.keys(cacheData).length > 0) {
                return structuredClone(cacheData);
            }
        }
        /* Params */
        const { all = true, id = '' } = params;
        /* Store data */
        const data = {};
        /* Single file */
        if ((0, string_js_1.isStringStrict)(id) && !all) {
            const file = await (0, promises_1.readFile)((0, node_path_1.resolve)(config_js_1.config.staticDir, `${id}.json`), { encoding: 'utf8' });
            const fileJson = (0, json_js_1.getJson)(file);
            if (fileJson !== undefined) {
                data[id] = fileJson;
            }
        }
        /* All files */
        if (!(0, string_js_1.isStringStrict)(id) && all) {
            const files = await (0, promises_1.readdir)((0, node_path_1.resolve)(config_js_1.config.staticDir));
            for (const file of files) {
                const fileExt = (0, node_path_1.extname)(file);
                const fileName = (0, node_path_1.basename)(file, fileExt);
                if (fileExt === '.json') {
                    const fileContents = await (0, promises_1.readFile)((0, node_path_1.resolve)(config_js_1.config.staticDir, file), { encoding: 'utf8' });
                    const fileJson = (0, json_js_1.getJson)(fileContents);
                    if (fileJson !== undefined) {
                        data[fileName] = fileJson;
                    }
                }
            }
        }
        /* Data not empty check */
        if (Object.keys(data).length === 0) {
            throw new Error('No file data');
        }
        /* Store in cache */
        if (config_js_1.config.env.cache) {
            const cacheDataFilterArgs = {
                key,
                type: 'set',
                data
            };
            await (0, filter_js_1.applyFilters)('cacheData', data, cacheDataFilterArgs);
        }
        /* Output */
        return data;
    }
    catch (error) {
        (0, print_js_1.print)('[SSF] Error getting file data', error);
        return {};
    }
};
exports.getFileData = getFileData;
/**
 * Get data from file system
 *
 * @param {AllFileDataArgs} args
 * @return {Promise<RenderAllData|undefined>}
 */
const getAllFileData = async (args) => {
    args = (0, object_js_1.isObjectStrict)(args) ? args : {};
    const { resolveProps = {
        image: ['image'],
        data: ['items', 'internalLink', 'parent']
    }, excludeProps = {
        data: ['content'],
        archive: {
            posts: ['content'],
            terms: ['content']
        }
    }, filterData, filterAllData } = args;
    try {
        /* Get data */
        let data = await getFileData('all_file_data', { all: true });
        /* Store all data */
        let allData = {
            navigation: [],
            navigationItem: [],
            redirect: [],
            content: {
                page: []
            }
        };
        config_js_1.config.partialTypes.forEach((contentType) => {
            allData[contentType] = [];
        });
        config_js_1.config.wholeTypes.forEach((contentType) => {
            allData.content[contentType] = [];
        });
        /* Data must be non empty object */
        if (!(0, object_js_1.isObjectStrict)(data) || Object.keys(data).length === 0) {
            throw new Error('No file data');
        }
        /* Image data */
        let imageData = {};
        if ((0, string_js_1.isStringStrict)(config_js_1.config.image.dataFile)) {
            const imageDataContents = await (0, promises_1.readFile)((0, node_path_1.resolve)(config_js_1.config.image.dataFile), { encoding: 'utf8' });
            const imageDataJson = (0, json_js_1.getJson)(imageDataContents);
            if (imageDataJson !== undefined) {
                imageData = imageDataJson;
            }
        }
        (0, fileDataNormal_js_1.resolveInternalLinks)(imageData, data, resolveProps.image);
        /* Id */
        Object.keys(data).forEach((d) => {
            const dataItem = data[d];
            if ((0, object_js_1.isObjectStrict)(dataItem)) {
                const { contentType } = dataItem;
                if ((0, string_js_1.isStringStrict)(contentType)) {
                    dataItem.id = d;
                }
            }
        });
        /* Internal props */
        resolveProps.data.forEach((d) => {
            (0, fileDataNormal_js_1.resolveInternalLinks)(data, data, [d], (prop, value) => {
                if (resolveProps.data.includes(prop)) {
                    const newValue = (0, fileDataNormal_js_1.undefineProps)(value, excludeProps.data);
                    return newValue;
                }
                return value;
            });
        });
        /* Filter data */
        if ((0, function_js_1.isFunction)(filterData)) {
            data = filterData(data);
        }
        /* Empty archive data */
        const archivePosts = {};
        // const archiveTerms: any = {}
        /* Set content */
        Object.keys(data).forEach((d) => {
            const dataItem = data[d];
            if (!(0, object_js_1.isObjectStrict)(dataItem)) {
                return;
            }
            const { contentType } = dataItem;
            if (!(0, string_js_1.isStringStrict)(contentType)) {
                return;
            }
            const partial = allData[contentType];
            if ((0, array_js_1.isArray)(partial)) {
                partial.push(dataItem);
            }
            const whole = allData.content[contentType];
            if ((0, array_js_1.isArray)(whole)) {
                whole.push(dataItem);
            }
            /* Archive */
            if (config_js_1.config.archiveMeta[contentType] !== undefined) {
                const dataItemCopy = (0, fileDataNormal_js_1.undefineProps)(dataItem, excludeProps.archive.posts);
                if (archivePosts[contentType] === undefined) {
                    archivePosts[contentType] = [];
                }
                archivePosts[contentType].push(dataItemCopy);
            }
        });
        /* Term content */
        /*
        Object.keys(config.contentTypes.taxonomy).forEach((tax) => {
          const { contentTypes, props } = config.contentTypes.taxonomy[tax]
    
          if (archiveTerms.terms[tax] === undefined) {
            archiveTerms.terms[tax] = {}
          }
    
          contentTypes.forEach((ct, i) => {
            const contentData = allData.content[ct]
    
            if (isArrayStrict(contentData)) {
              if (archiveTerms.terms[tax][ct] === undefined) {
                archiveTerms.terms[tax][ct] = {}
              }
    
              contentData.forEach((cd) => {
                const prop = props[i]
                const terms = cd[prop] as Generic[]
                const dataCopy = undefineProps(cd, excludeProps.archive.terms)
    
                if (isArrayStrict(terms)) {
                  terms.forEach((term) => {
                    if (!isObjectStrict(term)) {
                      return
                    }
    
                    const termId = term.id
    
                    if (!isStringStrict(termId)) {
                      return
                    }
    
                    if (archiveTerms[tax][ct][termId] === undefined) {
                      archiveTerms[tax][ct][termId] = []
                    }
    
                    const archiveTaxContentTypeTerm = archiveTerms[tax][ct][termId]
    
                    if (isArray(archiveTaxContentTypeTerm)) {
                      archiveTaxContentTypeTerm.push(dataCopy)
                    }
                  })
                }
              })
            }
          })
        })
        */
        /* Filter all data */
        if ((0, function_js_1.isFunction)(filterAllData)) {
            allData = filterAllData(allData);
        }
        /* Output */
        return allData;
    }
    catch (error) {
        (0, print_js_1.print)('[SSF] Error getting all file data', error);
        /* Output */
        return undefined;
    }
};
exports.getAllFileData = getAllFileData;
