"use strict";
/**
 * Utils - Filter
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.setFilters = exports.resetFilters = exports.applyFiltersSync = exports.applyFilters = exports.removeFilter = exports.addFilter = exports.filters = void 0;
const array_js_1 = require("../array/array.js");
const string_js_1 = require("../string/string.js");
const object_js_1 = require("../object/object.js");
const function_js_1 = require("../function/function.js");
/**
 * Store filter callbacks by name
 *
 * @type {FiltersFunctions}
 */
let filters = {
    columnProps: [],
    containerProps: [],
    fieldProps: [],
    formProps: [],
    richTextProps: [],
    richTextOutput: [],
    richTextContentItem: [],
    richTextContent: [],
    richTextContentOutput: [],
    renderItem: [],
    renderContent: [],
    renderContentStart: [],
    renderContentEnd: [],
    ajaxRes: [],
    cacheData: [],
    storeData: []
};
exports.filters = filters;
/**
 * Add filter to filters object
 *
 * @param {string} name
 * @param {function} filter
 * @return {boolean}
 */
const addFilter = (name, filter) => {
    if (!(0, string_js_1.isStringStrict)(name) || !(0, function_js_1.isFunction)(filter)) {
        return false;
    }
    if (filters[name] === undefined) {
        filters[name] = [];
    }
    filters[name].push(filter);
    return true;
};
exports.addFilter = addFilter;
/**
 * Remove filter from filters object
 *
 * @param {string} name
 * @param {function} filter
 * @return {boolean}
 */
const removeFilter = (name, filter) => {
    if (!(0, string_js_1.isStringStrict)(name) || !(0, function_js_1.isFunction)(filter)) {
        return false;
    }
    const callbacks = filters[name];
    if ((0, array_js_1.isArrayStrict)(callbacks)) {
        const index = callbacks.indexOf(filter);
        if (index > -1) {
            callbacks.splice(index, 1);
            return true;
        }
    }
    return false;
};
exports.removeFilter = removeFilter;
/**
 * Update value from callback return values
 *
 * @param {string} name
 * @param {*} value
 * @param {*} [args]
 * @return {Promise<*>}
 */
const applyFilters = async (name, value, args) => {
    const callbacks = filters[name];
    if ((0, array_js_1.isArrayStrict)(callbacks)) {
        for (const callback of callbacks) {
            value = await callback(value, args);
        }
    }
    return value;
};
exports.applyFilters = applyFilters;
/**
 * Synchronously update value from callback return values
 *
 * @param {string} name
 * @param {*} value
 * @param {*} [args]
 * @return {*}
 */
const applyFiltersSync = (name, value, args) => {
    const callbacks = filters[name];
    if ((0, array_js_1.isArrayStrict)(callbacks)) {
        for (const callback of callbacks) {
            value = callback(value, args);
        }
    }
    return value;
};
exports.applyFiltersSync = applyFiltersSync;
/**
 * Empty filters object
 *
 * @return {void}
 */
const resetFilters = () => {
    exports.filters = filters = {
        columnProps: [],
        containerProps: [],
        fieldProps: [],
        formProps: [],
        richTextProps: [],
        richTextOutput: [],
        richTextContentItem: [],
        richTextContent: [],
        richTextContentOutput: [],
        renderItem: [],
        renderContent: [],
        renderContentStart: [],
        renderContentEnd: [],
        ajaxRes: [],
        cacheData: [],
        storeData: []
    };
};
exports.resetFilters = resetFilters;
/**
 * Fill filters object
 *
 * @param {Filters} args
 * @return {boolean}
 */
const setFilters = (args) => {
    if (!(0, object_js_1.isObjectStrict)(args)) {
        return false;
    }
    const newFilters = Object.entries(args);
    if (newFilters.length === 0) {
        return false;
    }
    resetFilters();
    newFilters.forEach(([name, filter]) => {
        if (filter === undefined) {
            return;
        }
        addFilter(name, filter);
    });
    return true;
};
exports.setFilters = setFilters;
