"use strict";
/**
 * Utils - Image Static
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.setStaticImages = void 0;
const sharp_1 = __importDefault(require("sharp"));
const promises_1 = require("node:fs/promises");
const node_path_1 = require("node:path");
const config_js_1 = require("../../config/config.js");
const filePaths_js_1 = require("../file/filePaths.js");
const string_js_1 = require("../string/string.js");
const print_js_1 = require("../print/print.js");
/**
 * Get and transform static images (quality and sizes)
 *
 * @return {Promise<PromiseSettledResult<sharp.OutputInfo>[]>}
 */
const setStaticImages = async () => {
    const store = {};
    try {
        const inputDir = config_js_1.config.image.inputDir;
        const outputDir = config_js_1.config.image.outputDir;
        const dataFile = config_js_1.config.image.dataFile;
        if (!(0, string_js_1.isStringStrict)(inputDir) || !(0, string_js_1.isStringStrict)(outputDir)) {
            throw new Error('No input or output directories');
        }
        await (0, promises_1.mkdir)((0, node_path_1.resolve)(outputDir), { recursive: true });
        await (0, promises_1.mkdir)((0, node_path_1.resolve)((0, node_path_1.dirname)(dataFile)), { recursive: true });
        const sharpImages = [];
        for await (const path of (0, filePaths_js_1.getFilePaths)(inputDir)) {
            if (path.includes('.DS_Store')) {
                continue;
            }
            const ext = (0, node_path_1.extname)(path);
            const baseName = (0, node_path_1.basename)(path);
            const [base] = baseName.split(ext);
            /* Base required */
            if (base === undefined) {
                continue;
            }
            /* Nested files */
            let [, folders] = (0, node_path_1.dirname)(path).split(`${inputDir}/`);
            if (folders !== undefined && folders !== baseName) {
                await (0, promises_1.mkdir)((0, node_path_1.resolve)(outputDir, folders), { recursive: true });
            }
            else {
                folders = '';
            }
            /* Store meta data */
            const metadata = await (0, sharp_1.default)(path).metadata();
            const id = `${folders !== '' ? `${folders}/` : ''}${base}`;
            const [, format] = ext.split('.');
            if (format === undefined) {
                continue;
            }
            const { width = 0, height = 0, size: fileSize = 0, format: fileFormat = 'jpeg' } = metadata;
            store[id] = {
                path: id,
                name: baseName,
                size: fileSize,
                type: `image/${fileFormat}`,
                format,
                width,
                height
            };
            /* Sizes */
            let sizes = [...config_js_1.config.image.sizes];
            sizes.push(width);
            sizes = sizes.filter(s => s <= width);
            sizes.forEach((size) => {
                sharpImages.push({
                    size,
                    ext: format,
                    path: (0, node_path_1.resolve)(path),
                    newPath: (0, node_path_1.resolve)(outputDir, folders, `${base}${size !== width ? `@${size}` : ''}`)
                });
            });
        }
        if (sharpImages.length === 0) {
            throw new Error('No static images to transform');
        }
        if ((0, string_js_1.isStringStrict)(dataFile)) {
            await (0, promises_1.writeFile)((0, node_path_1.resolve)(dataFile), JSON.stringify(store));
        }
        return await Promise.allSettled(sharpImages.map(async (c) => {
            const { size, path, newPath, ext } = c;
            await (0, sharp_1.default)(path)
                .resize(size)
                .toFile(`${newPath}.${ext}`);
            const create = await (0, sharp_1.default)(path)
                .webp({ quality: config_js_1.config.image.quality })
                .resize(size)
                .toFile(`${newPath}.webp`);
            return create;
        }));
    }
    catch (error) {
        (0, print_js_1.print)('[SSF] Error transforming static images', error);
    }
    return [];
};
exports.setStaticImages = setStaticImages;
