"use strict";
/**
 * Utils - WordPress Data
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.getWordPressData = void 0;
const filter_js_1 = require("../filter/filter.js");
const object_js_1 = require("../object/object.js");
const array_js_1 = require("../array/array.js");
const string_js_1 = require("../string/string.js");
const config_js_1 = require("../../config/config.js");
const print_js_1 = require("../print/print.js");
const wordpressDataNormal_js_1 = require("./wordpressDataNormal.js");
/**
 * Fetch data from wordpress cms or cache
 *
 * @param {string} key
 * @param {string} route
 * @param {WordPressDataParams} params
 * @return {Promise<WordPressDataReturn>}
 */
const getWordPressData = async (key = '', route = '', params = {}) => {
    try {
        /* Key required for cache */
        if (!(0, string_js_1.isStringStrict)(key)) {
            throw new Error('No key');
        }
        /* Check cache */
        if (config_js_1.config.env.cache) {
            let cacheData = {};
            const cacheDataFilterArgs = {
                key,
                type: 'get'
            };
            cacheData = await (0, filter_js_1.applyFilters)('cacheData', cacheData, cacheDataFilterArgs);
            if ((0, object_js_1.isObject)(cacheData)) {
                return structuredClone(cacheData);
            }
        }
        /* Route required */
        if (!(0, string_js_1.isStringStrict)(route)) {
            throw new Error('No route');
        }
        /* Credentials */
        const { devUser, prodUser, devCredential, prodCredential, devHost, prodHost } = config_js_1.config.cms;
        let user = devUser;
        let pass = devCredential;
        let host = devHost;
        let status = 'any';
        if (config_js_1.config.env.prod) {
            user = prodUser;
            pass = prodCredential;
            host = prodHost;
            status = 'publish';
        }
        if (!(0, string_js_1.isStringStrict)(user) || !(0, string_js_1.isStringStrict)(pass) || !(0, string_js_1.isStringStrict)(host)) {
            throw new Error('No credentials');
        }
        /* Params */
        let url = `https://${host}/wp-json/wp/v2/${route}?_embed&status=${status}`;
        for (const [key, value] of Object.entries(params)) {
            url += `&${key}=${value.toString()}`;
        }
        /* Check and transform data */
        const resp = await fetch(url, {
            headers: {
                Authorization: `Basic ${btoa(`${user}:${pass}`)}`
            }
        });
        const data = await resp.json();
        /* Check if error */
        if (!(0, array_js_1.isArrayStrict)(data)) {
            throw new Error(data.message, { cause: data });
        }
        /* Normalize */
        const newItems = (0, wordpressDataNormal_js_1.normalizeWordPressData)(data);
        const newData = {
            items: newItems
        };
        /* Store in cache */
        if (config_js_1.config.env.cache) {
            const cacheDataFilterArgs = {
                key,
                type: 'set',
                data
            };
            await (0, filter_js_1.applyFilters)('cacheData', newData, cacheDataFilterArgs);
        }
        /* Output */
        return newData;
    }
    catch (error) {
        (0, print_js_1.print)('[SSF] Error fetching WordPress data', error);
        /* Store in cache (avoid extra calls when no result) */
        if (config_js_1.config.env.cache) {
            const cacheDataFilterArgs = {
                key,
                type: 'set',
                data: {}
            };
            await (0, filter_js_1.applyFilters)('cacheData', {}, cacheDataFilterArgs);
        }
        /* Output */
        return {};
    }
};
exports.getWordPressData = getWordPressData;
