"use strict";
/**
 * Utils - WordPress Data Normal
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.normalizeWordPressData = void 0;
const block_serialization_spec_parser_1 = require("@wordpress/block-serialization-spec-parser");
const objectUtils_js_1 = require("../object/objectUtils.js");
const string_js_1 = require("../string/string.js");
const array_js_1 = require("../array/array.js");
const object_js_1 = require("../object/object.js");
const number_js_1 = require("../number/number.js");
const config_js_1 = require("../../config/config.js");
/**
 * Reduce file props
 *
 * @private
 * @param {WordPressDataFile} file
 * @return {RenderFile}
 */
const _normalizeFile = (file) => {
    const { url, filename, alt, width, height, filesizeInBytes, subtype, type, sizes } = file;
    let s;
    if ((0, object_js_1.isObjectStrict)(sizes)) {
        s = {};
        for (const [, value] of Object.entries(sizes)) {
            const { width, url } = value;
            if (!(0, number_js_1.isNumber)(width) || !(0, string_js_1.isStringStrict)(url)) {
                continue;
            }
            s[width] = url;
        }
    }
    return {
        url: (0, string_js_1.isString)(url) ? url : '',
        name: (0, string_js_1.isString)(filename) ? filename : '',
        alt: (0, string_js_1.isString)(alt) ? alt : '',
        width: (0, number_js_1.isNumber)(width) ? width : 0,
        height: (0, number_js_1.isNumber)(height) ? height : 0,
        size: (0, number_js_1.isNumber)(filesizeInBytes) ? filesizeInBytes : 0,
        format: subtype,
        type: (0, string_js_1.isString)(type) ? type : '',
        sizes: s
    };
};
/**
 * Convert blocks to flatter props
 *
 * @private
 * @param {Block[]} blocks
 * @return {RenderItem[]}
 */
const _normalizeBlocks = (blocks) => {
    const newItems = [];
    /* Blocks must be array */
    if (!(0, array_js_1.isArrayStrict)(blocks)) {
        return [];
    }
    /* Recurse */
    blocks.forEach((block) => {
        if (!(0, object_js_1.isObjectStrict)(block)) {
            return;
        }
        const { blockName, attrs, innerBlocks } = block;
        const contentType = blockName;
        if (!(0, string_js_1.isStringStrict)(contentType)) {
            return;
        }
        const attributes = (0, object_js_1.isObjectStrict)(attrs) ? attrs : {};
        for (const [key, value] of Object.entries(attributes)) {
            if (!(0, object_js_1.isObjectStrict)(value)) {
                continue;
            }
            const attrContentType = value.contentType;
            if (attrContentType === 'file') {
                attributes[key] = _normalizeFile(value);
            }
        }
        const newItem = {
            contentType,
            ...attributes
        };
        if ((0, string_js_1.isString)(config_js_1.config.renderTypes[contentType])) {
            newItem.renderType = config_js_1.config.renderTypes[contentType];
        }
        if ((0, array_js_1.isArrayStrict)(innerBlocks)) {
            newItem.content = _normalizeBlocks(innerBlocks);
        }
        return newItems.push(newItem);
    });
    /* Output */
    return newItems;
};
/**
 * Transform item props to flatter structure
 *
 * @private
 * @param {WordPressDataItem} item
 * @return {RenderItem}
 */
const _normalizeItem = (item) => {
    const newItem = {};
    /* Loop item */
    for (const [key, value] of (0, objectUtils_js_1.getObjectEntries)(item)) {
        /* Value */
        let val = value;
        /* Check types */
        const isObj = (0, object_js_1.isObjectStrict)(value);
        /* Id */
        if (key === 'id') {
            val = value?.toString();
        }
        /* Content type */
        if (key === 'type' && (0, string_js_1.isStringStrict)(value)) {
            newItem.contentType = value;
            if ((0, string_js_1.isString)(config_js_1.config.renderTypes[value])) {
                newItem.renderType = config_js_1.config.renderTypes[value];
            }
            continue;
        }
        /* Flatten rendered */
        if (isObj) {
            const { rendered } = value;
            if ((0, string_js_1.isString)(rendered)) {
                val = rendered;
            }
        }
        /* Excerpt */
        if (key === 'excerpt' && (0, string_js_1.isString)(val)) {
            val = val.replace(/<(?:"[^"]*"['"]*|'[^']*'['"]*|[^'">])+>/g, '');
        }
        /* Content */
        if (key === 'content' && (0, string_js_1.isString)(val)) {
            val = _normalizeBlocks((0, block_serialization_spec_parser_1.parse)(val));
        }
        /* Embeded */
        if (key === '_embedded' && isObj) {
            const v = value;
            (0, objectUtils_js_1.getObjectKeys)(v).forEach((k) => {
                const embedVal = v[k];
                if (k === 'wp:term' && (0, array_js_1.isArrayStrict)(embedVal)) {
                    embedVal.forEach((e) => {
                        if ((0, array_js_1.isArrayStrict)(e)) {
                            e.forEach((ee) => {
                                if ((0, object_js_1.isObjectStrict)(ee)) {
                                    const { id, link, name, slug, taxonomy } = ee;
                                    const itemTaxonomy = item[taxonomy];
                                    if ((0, array_js_1.isArrayStrict)(itemTaxonomy)) {
                                        newItem[taxonomy] = itemTaxonomy.map((taxonomyId) => {
                                            if (taxonomyId === id) {
                                                return {
                                                    id,
                                                    link,
                                                    name,
                                                    slug,
                                                    taxonomy
                                                };
                                            }
                                            return taxonomyId;
                                        });
                                    }
                                }
                            });
                        }
                    });
                }
            });
        }
        /* Set value */
        newItem[key] = val;
    }
    /* Output */
    return newItem;
};
/**
 * Transform WordPressData data into simpler objects
 *
 * @param {WordPressDataItem[]} data
 * @param {RenderItem[]} [_newData]
 * @return {RenderItem[]}
 */
const normalizeWordPressData = (data, _newData = []) => {
    if (!(0, array_js_1.isArrayStrict)(data)) {
        return [];
    }
    /* Recurse data */
    data.forEach((item) => {
        if (!(0, object_js_1.isObjectStrict)(item)) {
            return;
        }
        _newData.push(_normalizeItem(item));
    });
    /* Output */
    return _newData;
};
exports.normalizeWordPressData = normalizeWordPressData;
